@@ -1,6 +1,7 @@
 
 from moviepy.editor import *
 from moviepy.video.fx import loop, mask_color, crop
+from bgeditor.common import utils
 from bgeditor.common.utils import cache_file, download_file, upload_file
 from bgeditor.dao.Lyric import Lyric
 from bgeditor.dao.Matrix import Matrix
@@ -8,6 +9,7 @@ from bgeditor.dao import MusicHelper
 from PIL import Image
 import numpy as np
 import requests, time
+from bgeditor.dao import FFmpeg
 from bgeditor.dao.FFmpeg import create_suource_can_loop_path, create_loop, create_loop_audio, create_video_audio, merge_intro_outro
 from proglog import ProgressBarLogger
 from bgeditor.common.utils import get_dir
@@ -53,37 +55,73 @@ class MyBarLogger(ProgressBarLogger):
 
 def create_video(list_comp_data, path_video, job_id, mf_server):
     print('get list')
+    intro_time=10
+    intro_time_real=0
     arr_comps=[]
     for comp_data in list_comp_data:
-        comp_data["job_id"]=job_id
+        comp_data["job_id"] = job_id
         comp_data["mf_server"] = mf_server
         arr_comps.append(Component.convert(comp_data))
     arr_comps.sort(key=lambda obj: obj.index)
     arr_composite = []
+    arr_composite_no_intro=[]
+    composite_intro_time=0
+    tmp_path_composite_intro = None
     max_duration = 0
     max_real_duration=1 # thoi gian cua 1 video
     is_compilation=False
     audio_compilation_path=None
     cnt_effect_vid=0
-    inoutroComp = None
+    introPath = None
+    outroPath = None
+    #check if videos are compilation
     for comp in arr_comps:
         if comp.type=="compilation":
             is_compilation=True
+            break
+
+    for comp in arr_comps:
+        if comp.type=="compilation":
             audio_compilation_path=comp.get_audio()
             continue
-        if comp.type=="inoutro":
-            inoutroComp = comp
-            continue
         if comp.type=="video":
             cnt_effect_vid += 1
+            is_cont=False
+            if comp.is_intro:
+                introPath = get_dir('coolbg_ffmpeg') + str(uuid.uuid4()) + "-intro.mp4"
+                rs = comp.get_clip()
+                if not rs.audio:
+                    rs.close()
+                    vidPathTmp = FFmpeg.add_null_sound(comp.video_path)
+                    rs = VideoFileClip(vidPathTmp)
+                rs.write_videofile(introPath, bitrate='4M', fps=24, codec='libx264', audio=False)
+                is_cont = True
+                rs.close()
+            if comp.is_outro:
+                outroPath = get_dir('coolbg_ffmpeg') + str(uuid.uuid4()) + "-outro.mp4"
+                rs = comp.get_clip()
+                if not rs.audio:
+                    rs.close()
+                    vidPathTmp = FFmpeg.add_null_sound(comp.video_path)
+                    rs = VideoFileClip(vidPathTmp)
+                rs.write_videofile(outroPath, bitrate='4M', fps=24, codec='libx264', audio=False)
+                is_cont = True
+                rs.close()
+            if is_cont:
+                continue
+
         if comp.type == "element":
-            #audio_tmp = CompositeVideoClip(arr_composite.copy()).audio
             comp.set_bg_clip(CompositeVideoClip(arr_composite.copy()))
-            #arr_composite = [comp.make().set_audio(audio_tmp)]
             arr_composite = [comp.make()]
         else:
-            arr_composite.append(comp.make())
-
+            compTmp = comp.make()
+            #cut tat ca comp nho hon 10s la intro
+            if is_compilation and comp.duration > 0 and comp.duration + comp.start_time <= intro_time:
+                if  comp.duration + comp.start_time > intro_time_real:
+                    intro_time_real = comp.duration + comp.start_time
+            else:
+                arr_composite_no_intro.append(compTmp)
+            arr_composite.append(compTmp)
         if comp.duration + comp.start_time > max_duration:
             max_duration = comp.duration + comp.start_time
         if comp.real_duration + comp.start_time > max_real_duration:
@@ -97,13 +135,20 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
     else:
         if audio_compilation_path is None:
             raise Exception("Audio Compilation Error!!!")
-        cnt_real_duration=0
+        cnt_real_duration=1
         while max_real_duration < 60*2/3 and cnt_real_duration<2 and cnt_effect_vid>1:
             max_real_duration*=2
             cnt_real_duration+=1
-        tmp_path = get_dir('coolbg_ffmpeg') + str(uuid.uuid4()) +"-final-vid.mp4"
+        #make intro composite
+        if intro_time_real > 0:
+            tmp_path_composite_intro = get_dir('coolbg_ffmpeg') + str(uuid.uuid4()) + "-comp-intro-vid.mp4"
+            tmp_clip = CompositeVideoClip(arr_composite).subclip(0, intro_time_real)
+            tmp_clip.write_videofile(tmp_path_composite_intro, bitrate='4M', fps=24, codec='libx264', audio=False, logger=logger)
+            tmp_clip.close()
+
+        tmp_path = get_dir('coolbg_ffmpeg') + str(uuid.uuid4()) + "-final-vid.mp4"
         tmp_path_maked_loop = get_dir('coolbg_ffmpeg') + str(uuid.uuid4()) + "-final-vid.mp4"
-        final_clip = CompositeVideoClip(arr_composite).subclip(0, max_real_duration)
+        final_clip = CompositeVideoClip(arr_composite_no_intro).subclip(0, max_real_duration)
         final_clip.write_videofile(tmp_path, bitrate='4M', fps=24, codec='libx264', audio=False, logger=logger)
         #crop big video
         # x_center=final_clip.w/2
@@ -119,7 +164,6 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
             if cross > 3:
                 cross = 3
             clip = make_loopable.make_loopable(final_clip, cross)
-            #clip.write_videofile(tmp_path_maked_loop, bitrate='4M', fps=24, codec='libx264', audio=False)
             clip.write_videofile(tmp_path_maked_loop, bitrate='4M', fps=24, codec='libx264', audio=False, logger=logger)
             clip.close()
             final_clip.close()
@@ -127,21 +171,17 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
             tmp_path_maked_loop = tmp_path
         audio_compilation = AudioFileClip(audio_compilation_path)
         final_clip_path = create_loop(tmp_path_maked_loop, audio_compilation.duration)
-        try:
-            os.remove(tmp_path_maked_loop)
-        except:
-            pass
-        try:
-            os.remove(tmp_path)
-        except:
-            pass
+        final_clip_path = merge_intro_outro(final_clip_path, tmp_path_composite_intro)
+
+        utils.remove(tmp_path_maked_loop)
+        utils.remove(tmp_path_composite_intro)
+        utils.remove(tmp_path)
         audio_compilation.close()
         create_video_audio(final_clip_path, audio_compilation_path, path_video)
     final_vid = VideoFileClip(path_video)
     duration_f = final_vid.duration
     final_vid.close()
-    if duration_f > 0 and inoutroComp is not None:
-        path_video = merge_intro_outro(path_video, inoutroComp.intro, inoutroComp.outro)
+    path_video = merge_intro_outro(path_video, introPath, outroPath)
     for comp in arr_comps:
         comp.close()
     return path_video
@@ -227,10 +267,10 @@ class Component:
 class InOutroComp(Component):
     def __init__(self, json_data):
         super().__init__(json_data)
-        self.intro =None
-        self.outro =None
+        self.intro = None
+        self.outro = None
         if json_data['intro'] and len(json_data['intro']) > 0:
-            self.intro =  download_file(json_data['intro'], ext=json_data['ext'])
+            self.intro = download_file(json_data['intro'], ext=json_data['ext'])
         if json_data['outro'] and len(json_data['outro']) > 0:
             self.outro = download_file(json_data['outro'], ext=json_data['ext'])
 
@@ -334,10 +374,16 @@ class VideoComp(Component):
         self.real_duration=0
         self.is_canva = json_data['isCanva']
         self.kind = "video"
+        self.is_intro=False
+        self.is_outro=False
+        if "is_intro" in json_data:
+            self.is_intro = json_data['is_intro']
+        if "is_outro" in json_data:
+            self.is_outro = json_data['is_outro']
         if self.is_canva:
             self.kind = "canva"
     def get_clip(self):
-        if self.isLoop and self.ext != "gif":
+        if self.isLoop and self.ext != "gif" and not self.is_intro and not self.is_outro:
             obj = requests.get(self.mf_server+"resource/get-md5/"+self.kind+"/"+self.md5).json()
             if "id" in obj:
                 if "loop_link" in obj and obj['loop_link'] is not None and "gdrive" in obj['loop_link']:
@@ -353,15 +399,19 @@ class VideoComp(Component):
                     self.video_path = path_loop
         else:
             self.video_path = download_file(self.video_url, ext=self.ext)
+        has_mask = False
+        if self.ext == "gif":
+            has_mask= True
+            self.video_path = FFmpeg.convert_gif(self.video_path)
 
-        rs = VideoFileClip(self.video_path, audio=not self.isMute)
-        self.real_duration=rs.duration
+        rs = VideoFileClip(self.video_path, audio=not self.isMute, has_mask=has_mask)
+        self.real_duration = rs.duration
         if (self.index == 0 or self.isLoop) and rs.duration < 1200: #max 20 mins loop
             rs.close()
             self.video_path = create_loop(self.video_path, 1200)
-            rs = VideoFileClip(self.video_path, audio=not self.isMute)
+            rs = VideoFileClip(self.video_path, audio=not self.isMute, has_mask=has_mask)
         if self.isMask:
-            rs = mask_color.mask_color(rs, self.maskColor, thr=30, s=3)
+            rs = mask_color.mask_color(rs, self.maskColor, thr=150, s=5)
         if self.opacity < 1.0:
             rs = rs.set_opacity(self.opacity)
         w, h = rs.size
